@@ -1,10 +1,6 @@
 package App::MiseEnPlace;
-{
-  $App::MiseEnPlace::VERSION = '0.15';
-}
-BEGIN {
-  $App::MiseEnPlace::AUTHORITY = 'cpan:GENEHACK';
-}
+our $AUTHORITY = 'cpan:GENEHACK';
+$App::MiseEnPlace::VERSION = '0.160';
 # ABSTRACT: A place for everything and everything in its place
 
 
@@ -15,56 +11,54 @@ use 5.010;
 use base 'App::Cmd::Simple';
 use autodie;
 use Carp;
-use File::Basename;
-use File::Path 2.08     qw/ make_path /;
-use File::Path::Expand;
-use Mouse;
+use File::HomeDir;
+use Path::Tiny;
 use Term::ANSIColor;
 use Try::Tiny;
-use YAML                qw/ LoadFile /;
+use Types::Standard -types;
+use YAML qw/ LoadFile /;
+
+use Moo;
+use MooX::HandlesVia;
 
 has bindir => (
-  is       => 'rw' ,
-  isa      => 'Str' ,
-  lazy     => 1 ,
-  default  => sub { expand_filename '~/bin/' } ,
+  is      => 'rw' ,
+  isa     => Str ,
+  lazy    => 1 ,
+  default => sub { path( File::HomeDir->my_home , 'bin' )->stringify } ,
 );
 
 has config_file => (
-  is       => 'rw' ,
-  isa      => 'Str' ,
-  lazy     => 1 ,
-  default  => "$ENV{HOME}/.mise" ,
+  is      => 'rw' ,
+  isa     => Str ,
+  lazy    => 1 ,
+  default => sub { path( File::HomeDir->my_home() , '.mise' )->stringify() } ,
 );
 
 has 'directories' => (
-  is      => 'rw' ,
-  isa     => 'ArrayRef[Str]' ,
-  traits  => [ 'Array' ] ,
-  handles => {
-    all_directories => 'elements' ,
-  } ,
+  is          => 'rw' ,
+  isa         => ArrayRef[Str] ,
+  handles_via => 'Array' ,
+  handles     => { all_directories => 'elements' } ,
 );
 
 has 'homedir' => (
-  is       => 'rw' ,
-  isa      => 'Str' ,
-  lazy     => 1 ,
-  default  => $ENV{HOME} ,
+  is      => 'rw' ,
+  isa     => Str ,
+  lazy    => 1 ,
+  default => sub { path( File::HomeDir->my_home )->stringify() } ,
 );
 
 has 'links' => (
-  is      => 'rw' ,
-  isa     => 'ArrayRef[ArrayRef[Str]]' ,
-  traits  => [ 'Array' ] ,
-  handles => {
-    all_links => 'elements' ,
-  } ,
+  is          => 'rw' ,
+  isa         => ArrayRef[ArrayRef[Str]] ,
+  handles_via => 'Array' ,
+  handles     => { all_links => 'elements' } ,
 );
 
 has 'verbose' => (
   is      => 'rw' ,
-  isa     => 'Bool' ,
+  isa     => Bool ,
   default => 0 ,
 );
 
@@ -88,7 +82,7 @@ sub validate_args {
   }
 
   $self->config_file( $opt->{config} ) if $opt->{config};
-  $self->verbose( $opt->{verbose} ) if $opt->{verbose};
+  $self->verbose( $opt->{verbose} )    if $opt->{verbose};
 }
 
 sub execute {
@@ -102,27 +96,14 @@ sub execute {
   # normal and nobody is the wiser.
   eval 'use IO::Page';
 
-  $self->_load_configs;
+  $self->_load_configs();
 
-  if ( $opt->{remove_bin_links} and -e -d $self->bindir ) {
-    my $bin = $self->bindir;
+  $self->_remove_bin_links($opt)
+    if $opt->{remove_bin_links} and -e -d $self->bindir();
 
-    opendir( my $dh , $bin );
-    while ( readdir $dh ) {
-      next unless -l "$bin/$_";
-
-      unlink "$bin/$_";
-
-      say colored('UNLINK' , 'bright_red' ) , " ~/bin/$_"
-        if $opt->{verbose};
-    }
-    closedir( $dh );
-  }
-
-  $self->_create_dir( $_ ) for $self->all_directories;
-
-  $self->_create_link( $_ ) for $self->all_links;
+  $self->_create_dir( $_ ) for $self->all_directories();
 
+  $self->_create_link( $_ ) for $self->all_links();
 }
 
 sub _create_dir {
@@ -130,21 +111,18 @@ sub _create_dir {
 
   my $msg;
 
-  no warnings 'experimental::smartmatch';
-  given( $dir ) {
-    when( -e -d ) {
-      $msg = colored('exists ','green') if $self->verbose;
-    }
-    when( -e and ! -l ) {
-      $msg = colored('ERROR: blocked by non-dirctory','bold white on_red');
-    }
-    default {
-      make_path $dir;
-      $msg = colored('created','bold black on_green');
-    }
+  if( -e -d $dir ) {
+    $msg = colored('exists ','green') if $self->verbose();
+  }
+  elsif( -e $dir and ! -l $dir ) {
+    $msg = colored('ERROR: blocked by non-dirctory','bold white on_red');
+  }
+  else {
+    path( $dir )->mkpath();
+    $msg = colored('created','bold black on_green');
   }
 
-  my $home = $self->homedir;
+  my $home = $self->homedir();
   if ( $msg ) {
     $dir =~ s/^$home/~/;
     say "[ DIR] $msg $dir";
@@ -178,8 +156,8 @@ sub _create_link {
     symlink $src , $target;
     $msg = colored( 'created' , 'bold black on_green' );
   }
-  my $home = $self->homedir;
 
+  my $home = $self->homedir();
   if ( $msg ) {
     $src    =~ s/^$home/~/;
     $target =~ s/^$home/~/;
@@ -190,25 +168,23 @@ sub _create_link {
 sub _load_configs {
   my( $self ) = shift;
 
-  unless ( -e $self->config_file ) {
-    say "Whoops, it looks like you don't have a ~/.mise file yet.";
+  unless ( -e $self->config_file() ) {
+    say "Whoops, it looks like you don't have a " . $self->config_file() . " file yet.";
     say "Please review the documentation, create one, and try again.";
     exit;
   }
 
-  my $base_config = _load_config_file( $self->config_file );
+  my $base_config = _load_config_file( $self->config_file() );
 
-  my @links = map { _parse_linkpair( $_ , $self->homedir) }
-    @{ $base_config->{create}{links} };
+  my @links = map { _parse_linkpair( $_ , $self->homedir() ) } @{ $base_config->{create}{links} };
 
-  my @dirs = map { _prepend_dir( $_ , $self->homedir) }
-    @{ $base_config->{create}{directories} };
+  my @dirs = map { _prepend_dir( $_ , $self->homedir() ) } @{ $base_config->{create}{directories} };
 
-  my @managed_dirs = map { glob _prepend_dir( $_ , $self->homedir ) }
-    @{ $base_config->{manage} };
+  my @managed_dirs = map { glob _prepend_dir( $_ , $self->homedir() ) } @{ $base_config->{manage} };
 
   for my $managed_dir ( @managed_dirs ) {
-    my $mise_file = "$managed_dir/.mise";
+    my $mise_file = path( $managed_dir , '.mise' )->stringify();
+
     if ( -e -r $mise_file ) {
       my $config = _load_config_file( $mise_file );
 
@@ -239,7 +215,6 @@ sub _load_config_file {
   };
 
   return $config;
-
 }
 
 sub _parse_create_links {
@@ -250,10 +225,11 @@ sub _parse_create_links {
   for my $link_pair ( @$link_array ) {
     my( $src , $target ) = ( %$link_pair );
 
-    my $src_base = basename( $src );
+    my $src_base = path( $src )->basename();
 
-    $target = $self->bindir if $target =~ m'BIN$';
-    $target = "$target$src_base" if $target =~ m|/$|;
+    $target = $self->bindir() if $target =~ m'BIN$';
+    $target = path($target, $src_base)->stringify()
+      if path($target)->is_dir() and ! path( $src )->is_dir();
 
     if (exists $link_targets{$target} ) {
       say "ERROR: Attempting to create multiple links to the same target:";
@@ -267,7 +243,6 @@ sub _parse_create_links {
   }
 
   return \@links;
-
 }
 
 sub _parse_linkpair {
@@ -290,9 +265,30 @@ sub _prepend_dir {
   confess "BAD ARGS" unless
     my( $base , $dir ) = @_;
 
-  return expand_filename $base if $base =~ m|^~|;
-  return "$dir/$base" unless $base =~ m|^/|;
-  return $base;
+  return path( $base )->stringify()        if $base =~ m|^~|;
+  return path( $dir )->stringify( )        unless $base;
+  return path( $dir , $base )->stringify() unless $base =~ m|^/|;
+  return path( $base )->stringify();
+}
+
+sub _remove_bin_links {
+  my( $self , $opt ) = @_;
+
+  my $bin = $self->bindir();
+
+  opendir( my $dh , $bin );
+  while ( readdir $dh ) {
+    my $path = path( $bin , $_ );
+
+    next unless -l $path;
+
+    $path->remove();
+
+    say colored('UNLINK' , 'bright_red' ) , " ~/bin/$_"
+      if $opt->{verbose};
+  }
+
+  closedir( $dh );
 }
 
 1;
@@ -301,13 +297,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 App::MiseEnPlace - A place for everything and everything in its place
 
 =head1 VERSION
 
-version 0.15
+version 0.160
 
 =head1 SYNOPSIS
 
@@ -319,7 +317,7 @@ John SJ Anderson <genehack@genehack.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by John SJ Anderson.
+This software is copyright (c) 2015 by John SJ Anderson.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
